import * as fs from "node:fs";
import * as path from "node:path";
import { saveFile, getFiles, exportPDF } from "./files.js";
import { parseFile } from "./parse.js";
import { getDate, getTitle, processGenerations } from "./process.js";
import { renderTable, renderTemplate } from "./render.js";

const query = process.argv.length > 2 ? process.argv[2].toLowerCase() : null;

const getFilenames = async (dataDirectoryPath, query) => {
  const files = await getFiles(dataDirectoryPath);
  return files.filter((filename) => !query || filename.includes(query));
};

const filenames = await getFilenames("data/", query);

for (const filename of filenames) {
  const basename = path.basename(filename).replace(".aml", "");
  const htmlFilename = `html/${basename}.html`;
  const pdfFilename = `pdf/${basename}.pdf`;

  const data = parseFile(filename);
  const { generations, numGenerations } = processGenerations(data);
  const table = renderTable(generations);

  const title = getTitle(data);
  const date = getDate();
  const bodyClass = `gens-${numGenerations}`;

  const outputData = renderTemplate({ title, date, table, bodyClass });

  saveFile(htmlFilename, outputData);

  exportPDF(`http://localhost/family-pedigrees/html/${htmlFilename}`, pdfFilename);

  console.log(pdfFilename);
}
