import archieml from "archieml";
import { loadFile } from "./files.js";

const parsePersonName = (line) => ({
  name: line.replace(/^< /, "").trim(),
  main: line.startsWith("< "),
});

const parsePersonLine = (personLine) => {
  if (!personLine.includes(" // ")) {
    return parsePersonName(personLine);
  }

  const parts = personLine.split(" // ");
  return {
    ...parsePersonName(parts[0]),
    life: parts[1].trim().replace("-", "–"),
  };
};

export const parseFamilyKey = (key) => {
  const parts = key.split(".");

  return {
    generation: parseInt(parts[0], 10),
    numInGen: parseInt(parts[1], 10),
  };
};

export const parseArchieMLString = (archieString) => {
  return archieml.load(archieString);
};

// Go through the raw ArchieML and process all the people lines
// Returns a list of families
export const parseRawData = (rawData) => {
  const output = [];

  for (const family of rawData.families) {
    output.push({
      ...family,
      ...parseFamilyKey(family.family),
      father: parsePersonLine(family.father),
      mother: parsePersonLine(family.mother),
      children: family.children.map((child) => ({
        ...child,
        ...parsePersonLine(child.name),
      })),
    });
  }

  return output;
};

export const parseFile = (filename) => {
  const data = loadFile(filename);
  const rawData = parseArchieMLString(data);

  return parseRawData(rawData);
};
