export const getTitle = (families) => {
  const firstFamily = families[0];

  return `${firstFamily.father.name} and ${firstFamily.mother.name} family pedigree`;
};

export const getDate = () => new Date().toISOString().slice(0, 10);

const totalGenerationsFromFamilies = (families) =>
  Math.max(...families.map((family) => family.generation));

const createEmptyGenerationsArray = (totalGenerations) => {
  const output = [];

  for (let gen = 0; gen < totalGenerations; gen++) {
    const numCells = Math.pow(2, gen);

    output.push(
      [...new Array(numCells)].map(() => ({
        empty: true,
      }))
    );
  }

  return output;
};

export const processGenerations = (families) => {
  const totalGenerations = totalGenerationsFromFamilies(families);
  const generationsArray = createEmptyGenerationsArray(totalGenerations);

  // Put each extant family in the right spot in the generations array
  for (const family of families) {
    const genIndex = family.generation - 1;
    const cellIndex = family.numInGen - 1;

    generationsArray[genIndex][cellIndex] = family;
  }

  return { generations: generationsArray, numGenerations: totalGenerations };
};
