import { loadFile } from "./files.js";

const renderLifespan = (lifespan) =>
  lifespan
    ? ` <span class="lifespan">${lifespan.replace("-", "&ndash;")}</span>`
    : "";

const renderPerson = (person) => `${person.name}${renderLifespan(person.life)}`;

const renderParents = (family) =>
  `<div class="parents">${renderPerson(family.father)} / ${renderPerson(
    family.mother
  )}</div>`;

const renderMarriage = (family) =>
  family.marriage ? `<div class="marriage">m. ${family.marriage}</div>` : "";

const renderChildMarriages = (child) =>
  child?.marriages
    ? `<span class="num-marriages">${child.marriages}</span>`
    : "";

const renderChildChildren = (child) =>
  child?.children ? `<span class="num-children">${child.children}</span>` : "";

const renderChild = (child) =>
  `<li${child.main ? ' class="main-line"' : ""}>${child.name}${renderLifespan(
    child.life
  )} ${renderChildMarriages(child)} ${renderChildChildren(child)}</li>`;

const renderChildren = (family) =>
  family.children.length > 0
    ? `<ul class="children">${family.children.map(renderChild).join("\n")}</ul>`
    : "";

const renderCellFromFamily = (
  family,
  totalGenerations
) => `<td rowspan="${Math.pow(
  2,
  totalGenerations - family.generation
)}" class="gen-${family.generation}">
${
  family.empty
    ? ""
    : `${renderParents(family)}
${renderMarriage(family)}
${renderChildren(family)}`
}
</td>`;

const renderRow = (cells) => `<tr>${cells.join("\n")}</tr>`;

export const renderTable = (generations) => {
  const numGens = generations.length;
  const numRows = Math.pow(2, numGens - 1);

  const rows = [];
  const familyCount = [...new Array(numGens)].fill(0);

  for (let rowIndex = 0; rowIndex < numRows; rowIndex++) {
    const cells = [];

    for (let gen = 0; gen < numGens; gen++) {
      if (rowIndex % Math.pow(2, numGens - gen - 1) === 0) {
        const cellIndex = familyCount[gen];

        cells.push(renderCellFromFamily(generations[gen][cellIndex], numGens));
        familyCount[gen] += 1;
      }
    }

    rows.push(renderRow(cells, numGens));
  }

  return rows.join("\n");
};

export const renderTemplate = ({ title, date, table, bodyClass }) => {
  const template = loadFile("template/index.html");

  return template
    .replace("{{ title }}", title)
    .replace("{{ date }}", date)
    .replace("{{ table }}", table)
    .replace("{{ bodyClass }}", bodyClass);
};
